@@ -6,6 +6,7 @@ import (
 	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
+	"path"
 	"strings"
 	"testing"
 
@@ -43,6 +44,37 @@ func TestCommentGitHubShowChangedProjects(t *testing.T) {
 		nil)
 }
 
+func TestCommentGitHubWithMissingGuardrailCheckPath(t *testing.T) {
+	GoldenFileCommandTest(t, testutil.CalcGoldenFileTestdataDirName(),
+		[]string{
+			"comment",
+			"github",
+			"--github-token", "abc",
+			"--repo", "test/test",
+			"--commit", "5",
+			"--show-changed",
+			"--path", "./testdata/changes.json",
+			"--dry-run",
+			"--guardrail-check-path", "./does/not/exist.json"},
+		nil)
+}
+
+func TestCommentGitHubWithGuardrailCheckPath(t *testing.T) {
+	dir := path.Join("./testdata", testutil.CalcGoldenFileTestdataDirName())
+	GoldenFileCommandTest(t, testutil.CalcGoldenFileTestdataDirName(),
+		[]string{
+			"comment",
+			"github",
+			"--github-token", "abc",
+			"--repo", "test/test",
+			"--commit", "5",
+			"--show-changed",
+			"--path", "./testdata/changes.json",
+			"--dry-run",
+			"--guardrail-check-path", path.Join(dir, "./guardrailCheck.json")},
+		nil)
+}
+
 var ghZeroCommentsResponse = `{ "data": { "repository": { "pullRequest": { "comments": { "nodes": [], "pageInfo": { "endCursor": "abc", "hasNextPage": false }}}}}}`
 var ghOneMatchingCommentResponse = `{ "data": { "repository": { "pullRequest": { "comments": { "nodes": [ 
             { "id": "123", "body": "infracomment body here, followed by tag: [//]: <> (infracost-comment)" }
