@@ -84,6 +84,57 @@ func TestDiffWithCompareTo(t *testing.T) {
 			RunTerraformCLI: true,
 		})
 }
+func TestDiffWithCompareToWithCurrentAndPastProjectError(t *testing.T) {
+	dir := path.Join("./testdata", testutil.CalcGoldenFileTestdataDirName())
+	GoldenFileCommandTest(
+		t,
+		testutil.CalcGoldenFileTestdataDirName(),
+		[]string{
+			"diff",
+			"--path",
+			dir,
+			"--compare-to",
+			path.Join(dir, "prior.json"),
+			"--format", "json",
+		}, &GoldenFileOptions{
+			IsJSON: true,
+		})
+}
+
+func TestDiffWithCompareToWithCurrentProjectError(t *testing.T) {
+
+	dir := path.Join("./testdata", testutil.CalcGoldenFileTestdataDirName())
+	GoldenFileCommandTest(
+		t,
+		testutil.CalcGoldenFileTestdataDirName(),
+		[]string{
+			"diff",
+			"--path",
+			dir,
+			"--compare-to",
+			path.Join(dir, "prior.json"),
+			"--format", "json",
+		}, &GoldenFileOptions{
+			IsJSON: true,
+		})
+}
+
+func TestDiffWithCompareToWithPastProjectError(t *testing.T) {
+	dir := path.Join("./testdata", testutil.CalcGoldenFileTestdataDirName())
+	GoldenFileCommandTest(
+		t,
+		testutil.CalcGoldenFileTestdataDirName(),
+		[]string{
+			"diff",
+			"--path",
+			dir,
+			"--compare-to",
+			path.Join(dir, "prior.json"),
+			"--format", "json",
+		}, &GoldenFileOptions{
+			IsJSON: true,
+		})
+}
 
 func TestDiffWithCompareToFormatJSON(t *testing.T) {
 	dir := path.Join("./testdata", testutil.CalcGoldenFileTestdataDirName())
@@ -104,6 +155,23 @@ func TestDiffWithCompareToFormatJSON(t *testing.T) {
 	)
 }
 
+func TestDiffWithCompareToPreserveSummary(t *testing.T) {
+	dir := path.Join("./testdata", testutil.CalcGoldenFileTestdataDirName())
+	GoldenFileCommandTest(
+		t,
+		testutil.CalcGoldenFileTestdataDirName(),
+		[]string{
+			"diff",
+			"--path",
+			"./testdata/express_route_gateway_plan.json",
+			"--compare-to",
+			path.Join(dir, "prior.json"),
+			"--show-skipped",
+		}, &GoldenFileOptions{
+			RunTerraformCLI: true,
+		})
+}
+
 func TestDiffWithInfracostJSON(t *testing.T) {
 	dir := path.Join("./testdata", testutil.CalcGoldenFileTestdataDirName())
 	GoldenFileCommandTest(
