@@ -13,8 +13,8 @@ import (
 	"time"
 
 	"github.com/Rhymond/go-money"
+	"github.com/hashicorp/go-multierror"
 	"github.com/pkg/errors"
-	"github.com/shopspring/decimal"
 	log "github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 	"golang.org/x/sync/errgroup"
@@ -28,7 +28,6 @@ import (
 	"github.com/infracost/infracost/internal/output"
 	"github.com/infracost/infracost/internal/prices"
 	"github.com/infracost/infracost/internal/providers"
-	"github.com/infracost/infracost/internal/providers/terraform"
 	"github.com/infracost/infracost/internal/schema"
 	"github.com/infracost/infracost/internal/ui"
 	"github.com/infracost/infracost/internal/usage"
@@ -45,11 +44,6 @@ type projectResult struct {
 	projectOut *projectOutput
 }
 
-type hclRunDiff struct {
-	resourceDiffs    map[string][]string
-	missingResources []string
-}
-
 func addRunFlags(cmd *cobra.Command) {
 	cmd.Flags().StringSlice("terraform-var-file", nil, "Load variable files, similar to Terraform's -var-file flag. Provided files must be relative to the --path flag")
 	cmd.Flags().StringSlice("terraform-var", nil, "Set value for an input variable, similar to Terraform's -var flag")
@@ -111,22 +105,16 @@ func runMain(cmd *cobra.Command, runCtx *config.RunContext) error {
 	projects := make([]*schema.Project, 0)
 	projectContexts := make([]*config.ProjectContext, 0)
 
-	hclProjects := make([]*schema.Project, 0)
 	for _, projectResult := range projectResults {
 		for _, project := range projectResult.projectOut.projects {
 			projectContexts = append(projectContexts, projectResult.ctx)
 			projects = append(projects, project)
 		}
-
-		hclProjects = append(hclProjects, projectResult.projectOut.hclProjects...)
 	}
 
-	wg := &sync.WaitGroup{}
-	var hclR *output.Root
-	if len(hclProjects) > 0 {
-		wg.Add(1)
-		hclR = new(output.Root)
-		go formatHCLProjects(wg, runCtx, hclProjects, hclR)
+	err = checkIfAllProjectsErrored(projects)
+	if err != nil {
+		return err
 	}
 
 	r, err := output.ToOutputFormat(projects)
@@ -141,7 +129,6 @@ func runMain(cmd *cobra.Command, runCtx *config.RunContext) error {
 		}
 	}
 
-	wg.Wait()
 	r.IsCIRun = runCtx.IsCIRun()
 	r.Currency = runCtx.Config.Currency
 	r.Metadata = output.NewMetadata(runCtx)
@@ -180,7 +167,7 @@ func runMain(cmd *cobra.Command, runCtx *config.RunContext) error {
 		runCtx.SetContextValue("lineCount", lines)
 	}
 
-	env := buildRunEnv(runCtx, projectContexts, r, projects, hclR, hclProjects)
+	env := buildRunEnv(runCtx, projectContexts, r)
 
 	pricingClient := apiclient.NewPricingAPIClient(runCtx)
 	err = pricingClient.AddEvent("infracost-run", env)
@@ -204,30 +191,48 @@ func runMain(cmd *cobra.Command, runCtx *config.RunContext) error {
 	return nil
 }
 
-func formatHCLProjects(wg *sync.WaitGroup, ctx *config.RunContext, hclProjects []*schema.Project, hclR *output.Root) {
-	defer func() {
-		err := recover()
-		wg.Done()
+func checkIfAllProjectsErrored(projects []*schema.Project) error {
+	allError := true
 
-		if err != nil {
-			err = apiclient.ReportCLIError(ctx, fmt.Errorf("hcl-runtime-error: formatting hcl projects %s\n%s", err, debug.Stack()), false)
-			if err != nil {
-				log.Debugf("error reporting unexpected hcl runtime error: %s", err)
-			}
-		}
-	}()
-
-	rr, err := output.ToOutputFormat(hclProjects)
-	if err != nil {
-		log.Debugf("could not format hcl project to root output")
+	if len(projects) == 0 {
+		allError = false
 	}
 
-	*hclR = rr
+	if len(projects) == 1 && len(projects[0].Metadata.Errors) == 1 {
+		return errors.New(projects[0].Metadata.Errors[0].Message)
+	}
+
+	for _, project := range projects {
+		if !project.Metadata.HasErrors() {
+			allError = false
+			break
+		}
+	}
+
+	if allError {
+		multi := &multierror.Error{}
+		for _, project := range projects {
+			msg := fmt.Sprintf("project %s errored: \n", project.Name)
+			for i, diag := range project.Metadata.Errors {
+				msg += "\t\t\t" + diag.Message
+
+				if i != len(project.Metadata.Errors)-1 {
+					msg += "\n"
+				}
+
+			}
+
+			multi.Errors = append(multi.Errors, errors.New(msg))
+		}
+
+		return multi
+	}
+
+	return nil
 }
 
 type projectOutput struct {
-	projects    []*schema.Project
-	hclProjects []*schema.Project
+	projects []*schema.Project
 }
 
 type parallelRunner struct {
@@ -316,7 +321,7 @@ func (r *parallelRunner) run() ([]projectResult, error) {
 				})
 				configProjects, err := r.runProjectConfig(ctx)
 				if err != nil {
-					return err
+					configProjects = newErroredProject(ctx, err)
 				}
 
 				projectResultChan <- projectResult{
@@ -469,13 +474,6 @@ func (r *parallelRunner) runProjectConfig(ctx *config.ProjectContext) (*projectO
 
 	usageData = usageFile.ToUsageDataMap()
 	out := &projectOutput{}
-	wg := &sync.WaitGroup{}
-
-	// if the provider is the dir provider let's run the hcl provider at the same time to get reporting metrics.
-	if _, ok := provider.(*terraform.DirProvider); ok {
-		wg.Add(1)
-		go r.runHCLProvider(wg, ctx, usageFile, out)
-	}
 
 	t1 := time.Now()
 	projects, err := provider.LoadResources(usageData)
@@ -532,9 +530,6 @@ func (r *parallelRunner) runProjectConfig(ctx *config.ProjectContext) (*projectO
 	taken := t2.Sub(t1).Milliseconds()
 	ctx.SetContextValue("tfProjectRunTimeMs", taken)
 
-	// wait for the hcl provider to finish if it hasn't already
-	wg.Wait()
-
 	spinner.Success()
 
 	if r.runCtx.Config.UsageActualCosts {
@@ -662,59 +657,6 @@ func (r *parallelRunner) populateActualCosts(projects []*schema.Project) {
 	}
 }
 
-func (r *parallelRunner) runHCLProvider(wg *sync.WaitGroup, ctx *config.ProjectContext, usageFile *usage.UsageFile, out *projectOutput) {
-	defer func() {
-		err := recover()
-		wg.Done()
-
-		if err != nil {
-			log.Debugf("recovered from hcl provider panic %s", err)
-			err = apiclient.ReportCLIError(r.runCtx, fmt.Errorf("hcl-runtime-error: loading resources %s\n%s", err, debug.Stack()), false)
-			if err != nil {
-				log.Debugf("error reporting unexpected hcl runtime error: %s", err)
-			}
-		}
-	}()
-	if r.runCtx.Config.DisableHCLParsing {
-		return
-	}
-
-	t1 := time.Now()
-
-	hclProvider, err := terraform.NewHCLProvider(ctx, &terraform.HCLProviderConfig{SuppressLogging: true})
-	if err != nil {
-		log.Debugf("Could not init HCL provider: %s", err)
-		return
-	}
-
-	projects, err := hclProvider.LoadResources(usageFile.ToUsageDataMap())
-	if err != nil {
-		log.Debugf("Error loading projects from HCL provider: %s", err)
-		return
-	}
-
-	for _, project := range projects {
-		err := prices.PopulatePrices(r.runCtx, project)
-		if err != nil {
-			log.Debugf("Error populating prices for HCL project: %s", err)
-			return
-		}
-		err = prices.PopulateActualCosts(r.runCtx, project)
-		if err != nil {
-			log.Debugf("Error populating usages for HCL project: %s", err)
-			return
-		}
-
-		schema.CalculateCosts(project)
-		project.CalculateDiff()
-	}
-
-	out.hclProjects = projects
-	t2 := time.Now()
-	taken := t2.Sub(t1).Milliseconds()
-	ctx.SetContextValue("hclProjectRunTimeMs", taken)
-}
-
 func (r *parallelRunner) generateUsageFile(ctx *config.ProjectContext, provider schema.Provider) error {
 	if ctx.ProjectConfig.UsageFile == "" {
 		// This should not happen as we check earlier in the code that usage-file is not empty when sync-usage-file flag is on.
@@ -957,7 +899,7 @@ func checkRunConfig(warningWriter io.Writer, cfg *config.Config) error {
 	return nil
 }
 
-func buildRunEnv(runCtx *config.RunContext, projectContexts []*config.ProjectContext, r output.Root, projects []*schema.Project, hclR *output.Root, hclProjects []*schema.Project) map[string]interface{} {
+func buildRunEnv(runCtx *config.RunContext, projectContexts []*config.ProjectContext, r output.Root) map[string]interface{} {
 	env := runCtx.EventEnvWithProjectContexts(projectContexts)
 
 	env["runId"] = r.RunID
@@ -988,11 +930,6 @@ func buildRunEnv(runCtx *config.RunContext, projectContexts []*config.ProjectCon
 	env["totalEstimatedUsages"] = summary.TotalEstimatedUsages
 	env["totalUnestimatedUsages"] = summary.TotalUnestimatedUsages
 
-	if hclR != nil {
-		AddHCLEnvVars(projectContexts, r, projects, *hclR, hclProjects, env)
-
-	}
-
 	if warnings := runCtx.GetResourceWarnings(); warnings != nil {
 		env["resourceWarnings"] = warnings
 	}
@@ -1004,137 +941,6 @@ func buildRunEnv(runCtx *config.RunContext, projectContexts []*config.ProjectCon
 	return env
 }
 
-// AddHCLEnvVars adds HCL reporting metrics to the Infracost run so that we can assess the accuracy of
-// the HCL approach.
-func AddHCLEnvVars(projectContexts []*config.ProjectContext, r output.Root, projects []*schema.Project, hclR output.Root, hclProjects []*schema.Project, env map[string]interface{}) {
-	var initialTotal decimal.Decimal
-	if r.TotalMonthlyCost != nil {
-		initialTotal = *r.TotalMonthlyCost
-	}
-
-	var hclTotal decimal.Decimal
-	if hclR.TotalMonthlyCost != nil {
-		hclTotal = *hclR.TotalMonthlyCost
-	}
-
-	env["hclPercentChange"] = "0.00"
-	env["absHclPercentChange"] = "0.00"
-	change := percentChange(hclTotal, initialTotal)
-	abs := change.Abs()
-
-	if abs.GreaterThan(decimal.NewFromInt(0)) {
-		env["hclPercentChange"] = change.StringFixed(2)
-		env["absHclPercentChange"] = abs.StringFixed(2)
-	}
-
-	for _, k := range os.Environ() {
-		if strings.HasPrefix(k, "TF_VAR") {
-			env["tfVarPresent"] = true
-			break
-		}
-	}
-
-	var tfTimeTaken int64
-	var hclTimeTaken int64
-	for _, pCtx := range projectContexts {
-		if v, ok := pCtx.ContextValues()["tfProjectRunTimeMs"]; ok {
-			tfTimeTaken += v.(int64)
-		}
-
-		if v, ok := pCtx.ContextValues()["hclProjectRunTimeMs"]; ok {
-			hclTimeTaken += v.(int64)
-		}
-	}
-
-	env["tfRunTimeMs"] = tfTimeTaken
-	env["hclRunTimeMs"] = hclTimeTaken
-
-	diff := collectHCLRunDiff(projects, hclProjects)
-	env["hclMissingResources"] = diff.missingResources
-	env["hclResourceDiff"] = diff.resourceDiffs
-}
-
-func collectHCLRunDiff(projects, hclProjects []*schema.Project) hclRunDiff {
-	diff := map[string][]string{}
-	missingResources := map[string]bool{}
-
-	hclProjectsMapping := map[string]*schema.Project{}
-	for _, project := range hclProjects {
-		hclProjectsMapping[project.Name] = project
-	}
-
-	for _, project := range projects {
-		hclProject := hclProjectsMapping[project.Name]
-
-		if hclProject == nil {
-			log.Debugf("could not find a matching HCL project '%s' for HCL run diff", project.Name)
-			continue
-		}
-
-		hclResourcesMapping := map[string]*decimal.Decimal{}
-
-		for _, hclResource := range hclProject.Resources {
-			hclResourcesMapping[hclResource.Name] = hclResource.MonthlyCost
-		}
-
-		for _, resource := range project.Resources {
-			hclResourceCost, ok := hclResourcesMapping[resource.Name]
-			if !ok {
-				missingResources[resource.ResourceType] = true
-				continue
-			}
-
-			if resource.MonthlyCost == nil && hclResourceCost == nil {
-				continue
-			}
-
-			hclCost := decimal.NewFromInt(0)
-			if hclResourceCost != nil {
-				hclCost = *hclResourceCost
-			}
-
-			var cost decimal.Decimal
-			change := decimal.NewFromInt(100)
-			abs := decimal.NewFromInt(100)
-
-			if resource.MonthlyCost != nil {
-				cost = *resource.MonthlyCost
-
-				change = percentChange(hclCost, cost)
-				abs = change.Abs()
-			}
-
-			if abs.GreaterThan(decimal.NewFromInt(0)) {
-				if diff[resource.ResourceType] == nil {
-					diff[resource.ResourceType] = []string{}
-				}
-
-				diff[resource.ResourceType] = append(diff[resource.ResourceType], change.StringFixed(2))
-			}
-		}
-	}
-
-	missingList := make([]string, 0, len(missingResources))
-	for key := range missingResources {
-		missingList = append(missingList, key)
-	}
-
-	runDiff := hclRunDiff{
-		missingResources: missingList,
-		resourceDiffs:    diff,
-	}
-
-	return runDiff
-}
-
-func percentChange(a decimal.Decimal, b decimal.Decimal) decimal.Decimal {
-	if b.IsZero() {
-		return decimal.NewFromInt(0)
-	}
-
-	return a.Sub(b).Div(b).Mul(decimal.NewFromInt(100))
-}
-
 func unwrapped(err error) error {
 	e := err
 	for errors.Unwrap(e) != nil {
@@ -1143,3 +949,16 @@ func unwrapped(err error) error {
 
 	return e
 }
+
+func newErroredProject(ctx *config.ProjectContext, err error) *projectOutput {
+	metadata := config.DetectProjectMetadata(ctx.ProjectConfig.Path)
+	metadata.Type = "error"
+	metadata.AddError(err)
+
+	name := ctx.ProjectConfig.Name
+	if name == "" {
+		name = metadata.GenerateProjectName(ctx.RunContext.VCSMetadata.Remote, ctx.RunContext.IsCloudEnabled())
+	}
+
+	return &projectOutput{projects: []*schema.Project{schema.NewProject(name, metadata)}}
+}
