@@ -98,7 +98,13 @@ func (b referencedBlocks) Less(i, j int) bool {
 	return false
 }
 
-// ModuleBlocks returns all the Blocks of type module. The returned Blocks
+// ModuleBlocks is a wrapper around SortedByCaller that selects just Modules to be sorted.
+func (blocks Blocks) ModuleBlocks() Blocks {
+	justModules := blocks.OfType("module")
+	return justModules.SortedByCaller()
+}
+
+// SortedByCaller returns all the Blocks of type module. The returned Blocks
 // are sorted in order of reference. Blocks that are referenced by others are
 // the first in this list.
 //
@@ -107,17 +113,15 @@ func (b referencedBlocks) Less(i, j int) bool {
 //
 // This makes the list returned safe for context evaluation, as we evaluate modules that have
 // outputs that other modules rely on first.
-func (blocks Blocks) ModuleBlocks() Blocks {
-	justModules := blocks.OfType("module")
-	toSort := make(referencedBlocks, len(justModules))
-
-	copy(toSort, justModules)
+func (blocks Blocks) SortedByCaller() Blocks {
+	sorted := make(Blocks, len(blocks))
+	toSort := make(referencedBlocks, len(blocks))
 
+	copy(toSort, blocks)
 	sort.Sort(toSort)
+	copy(sorted, toSort)
 
-	copy(justModules, toSort)
-
-	return justModules
+	return sorted
 }
 
 // Blocks is a helper type around a slice of blocks to provide easy access
@@ -240,14 +244,18 @@ type Block struct {
 	expanded bool
 	// cloneIndex represents the index of the parent that this Block has been cloned from
 	cloneIndex int
+	// parent is the block that the block was cloned from
+	parent *Block
 	// childBlocks holds information about any child Blocks that the Block may have. This can be empty.
 	// See Block docs for more information about child Blocks.
 	childBlocks Blocks
 	// verbose determines whether the block uses verbose debug logging.
-	verbose  bool
-	newMock  func(attr *Attribute) cty.Value
+	verbose    bool
+	logger     *logrus.Entry
+	newMock    func(attr *Attribute) cty.Value
+	attributes []*Attribute
+
 	Filename string
-	logger   *logrus.Entry
 }
 
 // BlockBuilder handles generating new Blocks as part of the parsing and evaluation process.
@@ -365,6 +373,7 @@ func (b BlockBuilder) CloneBlock(block *Block, index cty.Value) *Block {
 	clone.expanded = true
 	block.cloneIndex++
 
+	clone.parent = block
 	return clone
 }
 
@@ -500,6 +509,10 @@ func (b *Block) IsForEachReferencedExpanded(moduleBlocks Blocks) bool {
 	}
 
 	label := r.String()
+	if blockType == "module" {
+		label = r.typeLabel
+	}
+
 	referenced := moduleBlocks.Matching(BlockMatcher{
 		Type:       blockType,
 		Label:      label,
@@ -510,7 +523,7 @@ func (b *Block) IsForEachReferencedExpanded(moduleBlocks Blocks) bool {
 		return true
 	}
 
-	return referenced.IsCountExpanded()
+	return !referenced.ShouldExpand()
 }
 
 func (b Block) ShouldExpand() bool {
@@ -518,13 +531,25 @@ func (b Block) ShouldExpand() bool {
 		return false
 	}
 
-	return b.Type() == "resource" || b.Type() == "module" || b.Type() == "data"
+	validType := b.Type() == "resource" || b.Type() == "module" || b.Type() == "data"
+	if !validType {
+		return false
+	}
+
+	countAttr := b.GetAttribute("count")
+	forEachAttr := b.GetAttribute("for_each")
+
+	return countAttr != nil || forEachAttr != nil
 }
 
 // SetContext sets the Block.context to the provided ctx. This ctx is also set on the child Blocks as
 // a child Context. Meaning that it can be used in traversal evaluation when looking up Context variables.
 func (b *Block) SetContext(ctx *Context) {
 	b.context = ctx
+	for _, attribute := range b.attributes {
+		attribute.Ctx = ctx
+	}
+
 	for _, block := range b.childBlocks {
 		block.SetContext(ctx.NewChild())
 	}
@@ -702,13 +727,18 @@ func (b *Block) Children() Blocks {
 //
 // ami & instance_type are the Attributes of this Block and credit_specification is a child Block.
 func (b *Block) GetAttributes() []*Attribute {
-	var results []*Attribute
 	if b == nil || b.hclBlock == nil {
 		return nil
 	}
 
-	for _, attr := range b.getHCLAttributes() {
-		results = append(results, &Attribute{
+	if b.attributes != nil {
+		return b.attributes
+	}
+
+	hclAttributes := b.getHCLAttributes()
+	var attributes = make([]*Attribute, 0, len(hclAttributes))
+	for _, attr := range hclAttributes {
+		attributes = append(attributes, &Attribute{
 			newMock: b.newMock,
 			HCLAttr: attr,
 			Ctx:     b.context,
@@ -719,7 +749,8 @@ func (b *Block) GetAttributes() []*Attribute {
 		})
 	}
 
-	return results
+	b.attributes = attributes
+	return attributes
 }
 
 // GetAttribute returns the given attribute with the provided name. It will return nil if the attribute is not found.
@@ -800,6 +831,10 @@ func (b *Block) Values() cty.Value {
 	values := make(map[string]cty.Value)
 
 	for _, attribute := range b.GetAttributes() {
+		if attribute.Name() == "for_each" {
+			continue
+		}
+
 		values[attribute.Name()] = attribute.Value()
 	}
 
