@@ -38,6 +38,8 @@ var (
 		"dynamic":  {},
 		"data":     {},
 	}
+
+	sensitiveRegxp = regexp.MustCompile(`_token$|^token_|_secret$|^secret_|_password$|^password_|_username$|^username_|api_key|expiration_date`)
 )
 
 const maxContextIterations = 120
@@ -55,7 +57,7 @@ type Evaluator struct {
 	// HCL attributes.
 	inputVars map[string]cty.Value
 	// moduleCalls are the modules that the list of Blocks call to. This is built at runtime.
-	moduleCalls []*ModuleCall
+	moduleCalls map[string]*ModuleCall
 	// moduleMetadata is a lookup map of where modules exist on the local filesystem. This is built as part of a
 	// Terraform or Infracost init.
 	moduleMetadata *modules.Manifest
@@ -132,6 +134,7 @@ func NewEvaluator(
 		ctx:            ctx,
 		inputVars:      inputVars,
 		moduleMetadata: moduleMetadata,
+		moduleCalls:    map[string]*ModuleCall{},
 		visitedModules: visitedModules,
 		workspace:      workspace,
 		workingDir:     workingDir,
@@ -147,9 +150,28 @@ func (e *Evaluator) MissingVars() []string {
 
 	blocks := e.module.Blocks.OfType("variable")
 	for _, block := range blocks {
+		name := block.Label()
+
+		var sensitive bool
+		value := block.GetAttribute("sensitive").Value()
+		if !value.IsNull() {
+			err := gocty.FromCtyValue(value, &sensitive)
+			if err != nil {
+				e.logger.Debugf("could not convert 'sensitive' attribute for variable.%s err: %s", name, err)
+			}
+		}
+
+		if sensitive {
+			continue
+		}
+
+		if sensitiveRegxp.MatchString(name) {
+			continue
+		}
+
 		_, v := e.evaluateVariable(block)
 		if v == errorNoVarValue {
-			missing = append(missing, fmt.Sprintf("variable.%s", block.Label()))
+			missing = append(missing, fmt.Sprintf("variable.%s", name))
 		}
 	}
 
@@ -172,7 +194,7 @@ func (e *Evaluator) Run() (*Module, error) {
 	e.evaluate(lastContext)
 
 	// let's load the modules now we have our top level context.
-	e.moduleCalls = e.loadModules(lastContext)
+	e.loadModules(lastContext)
 	e.logger.Debug("evaluating context after loading modules")
 	e.evaluate(lastContext)
 
@@ -291,7 +313,13 @@ func (e *Evaluator) evaluateModules() {
 
 		moduleCall.Module, _ = moduleEvaluator.Run()
 		outputs := moduleEvaluator.exportOutputs()
-		e.ctx.Set(outputs, "module", moduleCall.Name)
+		if v := moduleCall.Module.Key(); v != nil {
+			e.ctx.Set(outputs, "module", stripCount(moduleCall.Name), *v)
+		} else if v := moduleCall.Module.Index(); v != nil {
+			e.ctx.Set(outputs, "module", stripCount(moduleCall.Name), fmt.Sprintf("%d", *v))
+		} else {
+			e.ctx.Set(outputs, "module", moduleCall.Name)
+		}
 	}
 }
 
@@ -341,7 +369,7 @@ func (e *Evaluator) expandDynamicBlock(b *Block) {
 	}
 
 	for _, sub := range b.Children().OfType("dynamic") {
-		e.logger.Debugf("expanding block %s because a dynamic blocka was found %s", b.LocalName(), sub.LocalName())
+		e.logger.Debugf("expanding block %s because a dynamic block was found %s", b.LocalName(), sub.LocalName())
 
 		blockName := sub.TypeLabel()
 		expanded := e.expandBlockForEaches([]*Block{sub})
@@ -395,14 +423,25 @@ func (e *Evaluator) expandDynamicBlock(b *Block) {
 // will expand correctly to: aws_eip.nat_gateway[a], aws_eip.nat_gateway[b]. Rather than aws_eip.nat_gateway[id], aws_eip.nat_gateway[a] ...
 func (e *Evaluator) expandBlockForEaches(blocks Blocks) Blocks {
 	var expanded Blocks
+	var haveChanged = make(map[string]*Block)
 	for _, block := range blocks {
 		forEachAttr := block.GetAttribute("for_each")
-
-		if forEachAttr == nil || block.IsCountExpanded() || !block.IsForEachReferencedExpanded(blocks) || !shouldExpandBlock(block) {
+		if forEachAttr == nil {
 			expanded = append(expanded, block)
 			continue
 		}
 
+		if block.IsCountExpanded() || !block.IsForEachReferencedExpanded(blocks) || !shouldExpandBlock(block) {
+			parent := block.parent.GetAttribute("for_each")
+			if !parent.HasChanged() {
+				expanded = append(expanded, block)
+			} else {
+				haveChanged[block.parent.FullName()] = block.parent
+			}
+
+			continue
+		}
+
 		e.logger.Debugf("expanding block %s because a for_each attribute was found", block.LocalName())
 
 		value := forEachAttr.Value()
@@ -416,6 +455,9 @@ func (e *Evaluator) expandBlockForEaches(blocks Blocks) Blocks {
 				nameLabel = block.TypeLabel()
 			}
 
+			// set the context to an empty object so that we don't have any of the prior unexpanded attributes
+			// like id/arn e.t.c polluting the expansion. Otherwise we can end up with lots of expanded resources
+			// with attribute names e.g. aws_instance.test[id], aws_instance.test[arn].
 			e.ctx.Set(cty.ObjectVal(make(map[string]cty.Value)), typeLabel, nameLabel)
 
 			value.ForEachElement(func(key cty.Value, val cty.Value) bool {
@@ -437,6 +479,9 @@ func (e *Evaluator) expandBlockForEaches(blocks Blocks) Blocks {
 				ctx.Set(key, block.TypeLabel(), "key")
 				ctx.Set(val, block.TypeLabel(), "value")
 
+				if v, ok := e.moduleCalls[clone.FullName()]; ok {
+					v.Definition = clone
+				}
 				expanded = append(expanded, clone)
 
 				return false
@@ -444,6 +489,16 @@ func (e *Evaluator) expandBlockForEaches(blocks Blocks) Blocks {
 		}
 	}
 
+	if len(haveChanged) > 0 {
+		var changes = make(Blocks, 0, len(haveChanged))
+		for _, block := range haveChanged {
+			changes = append(changes, block)
+		}
+
+		eaches := e.expandBlockForEaches(changes)
+		return append(expanded, eaches...)
+	}
+
 	return expanded
 }
 
@@ -763,13 +818,25 @@ func (e *Evaluator) loadModule(b *Block) (*ModuleCall, error) {
 }
 
 // loadModules reads all module blocks and loads the underlying modules, adding blocks to moduleCalls.
-func (e *Evaluator) loadModules(lastContext hcl.EvalContext) []*ModuleCall {
+func (e *Evaluator) loadModules(lastContext hcl.EvalContext) {
 	e.logger.Debug("loading module calls")
-	var moduleDefinitions []*ModuleCall
+
+	var moduleBlocks Blocks
+	var filtered Blocks
+	for _, block := range e.module.Blocks {
+		if block.Type() == "module" {
+			moduleBlocks = append(moduleBlocks, block)
+		} else {
+			// remove the block from the top level blocks as we'll replace these with expanded blocks.
+			filtered = append(filtered, block)
+		}
+	}
+
+	expanded := e.expandBlocks(moduleBlocks.SortedByCaller(), lastContext)
+	filtered = append(filtered, expanded...)
+	e.module.Blocks = filtered
 
 	// TODO: if a module uses a count that depends on a module output, then the block expansion might be incorrect.
-	expanded := e.expandBlocks(e.module.Blocks.ModuleBlocks(), lastContext)
-
 	for _, moduleBlock := range expanded {
 		if moduleBlock.Label() == "" {
 			continue
@@ -781,10 +848,8 @@ func (e *Evaluator) loadModules(lastContext hcl.EvalContext) []*ModuleCall {
 			continue
 		}
 
-		moduleDefinitions = append(moduleDefinitions, moduleCall)
+		e.moduleCalls[moduleBlock.FullName()] = moduleCall
 	}
-
-	return moduleDefinitions
 }
 
 // expFunctions returns the set of functions that should be used to when evaluating
