@@ -13,6 +13,7 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"github.com/zclconf/go-cty/cty"
+	ctyJson "github.com/zclconf/go-cty/cty/json"
 
 	"github.com/infracost/infracost/internal/hcl/modules"
 	"github.com/infracost/infracost/internal/sync"
@@ -52,7 +53,7 @@ data "cats_cat" "the-cats-mother" {
 
 	logger := newDiscardLogger()
 	loader := modules.NewModuleLoader(filepath.Dir(path), nil, logger, &sync.KeyMutex{})
-	parsers, err := LoadParsers(filepath.Dir(path), loader, nil, logger, OptionStopOnHCLError())
+	parsers, err := LoadParsers(filepath.Dir(path), loader, nil, logger)
 	require.NoError(t, err)
 	module, err := parsers[0].ParseDirectory()
 	require.NoError(t, err)
@@ -560,7 +561,7 @@ output "mod_result" {
 	logger := newDiscardLogger()
 	dir := filepath.Dir(path)
 	loader := modules.NewModuleLoader(dir, nil, logger, &sync.KeyMutex{})
-	parsers, err := LoadParsers(path, loader, nil, logger, OptionStopOnHCLError())
+	parsers, err := LoadParsers(path, loader, nil, logger)
 	require.NoError(t, err)
 	rootModule, err := parsers[0].ParseDirectory()
 	require.NoError(t, err)
@@ -620,7 +621,7 @@ output "mod_result" {
 
 	logger := newDiscardLogger()
 	loader := modules.NewModuleLoader(filepath.Dir(path), nil, logger, &sync.KeyMutex{})
-	parsers, err := LoadParsers(path, loader, nil, logger, OptionStopOnHCLError())
+	parsers, err := LoadParsers(path, loader, nil, logger)
 	require.NoError(t, err)
 	rootModule, err := parsers[0].ParseDirectory()
 	require.NoError(t, err)
@@ -817,7 +818,7 @@ resource "test_resource_two" "test" {
 
 	logger := newDiscardLogger()
 	loader := modules.NewModuleLoader(filepath.Dir(path), nil, logger, &sync.KeyMutex{})
-	parsers, err := LoadParsers(filepath.Dir(path), loader, nil, logger, OptionStopOnHCLError())
+	parsers, err := LoadParsers(filepath.Dir(path), loader, nil, logger)
 	require.NoError(t, err)
 	module, err := parsers[0].ParseDirectory()
 	require.NoError(t, err)
@@ -856,13 +857,13 @@ locals {
 module "test" {
     for_each = local.az
 	source = "../."
-	input = "ok"
+	input = each.value.bz
 }
 
 resource "test_two" "test" {
   for_each        = module.test 
   inherited_id    = each.value.id
-  inherited_attr  = each.value.another_attr
+  inherited_attr  = each.value.mod_result
 }
 `,
 		`
@@ -879,29 +880,187 @@ output "mod_result" {
 
 	logger := newDiscardLogger()
 	loader := modules.NewModuleLoader(filepath.Dir(path), nil, logger, &sync.KeyMutex{})
-	parsers, err := LoadParsers(path, loader, nil, logger, OptionStopOnHCLError())
+	parsers, err := LoadParsers(path, loader, nil, logger)
 	require.NoError(t, err)
 	module, err := parsers[0].ParseDirectory()
 	require.NoError(t, err)
 
 	blocks := module.Blocks
 	resources := blocks.OfType("resource")
-	labels := make([]string, len(resources))
-	for i, resource := range resources {
-		labels[i] = resource.Label()
+
+	var ar *Block
+	var br *Block
+	for _, resource := range resources {
+		if resource.Label() == "test_two.test[\"a\"]" {
+			ar = resource
+		}
+
+		if resource.Label() == "test_two.test[\"b\"]" {
+			br = resource
+		}
 	}
-	assert.ElementsMatch(t, []string{
-		`test_two.test["a"]`,
-		`test_two.test["b"]`,
-	}, labels)
+	require.NotNil(t, ar, "test_two.test[\"a\"] was not found in module blocks")
+	require.NotNil(t, br, "test_two.test[\"b\"] was not found in module blocks")
+
+	s := ar.GetAttribute("inherited_attr").AsString()
+	assert.Equal(t, "w", s)
+
+	s = br.GetAttribute("inherited_attr").AsString()
+	assert.Equal(t, "z", s)
 
 	modules := blocks.OfType("module")
 	modLabels := make([]string, len(modules))
 	for i, module := range modules {
 		modLabels[i] = "module." + module.Label()
 	}
+
 	assert.ElementsMatch(t, []string{
 		`module.test["a"]`,
 		`module.test["b"]`,
 	}, modLabels)
+
+	var a *Module
+	for _, m := range module.Modules {
+		if m.Name == `module.test["a"]` {
+			a = m
+		}
+	}
+	require.NotNil(t, a, "could not find module.test[a] in root module")
+	out := a.Blocks.Outputs(false)
+	v := out.AsValueMap()
+	s = v["mod_result"].AsString()
+	assert.Equal(t, "w", s)
+
+	var b *Module
+	for _, m := range module.Modules {
+		if m.Name == `module.test["b"]` {
+			b = m
+		}
+	}
+	require.NotNil(t, b, "could not find module.test[b] in root module")
+	out = b.Blocks.Outputs(false)
+	v = out.AsValueMap()
+	s = v["mod_result"].AsString()
+	assert.Equal(t, "z", s)
+}
+
+func Test_ModuleExpansionBehindComplexExpression(t *testing.T) {
+	path := createTestFileWithModule(`
+variable "var1" {
+  default = {
+	"foo" = {
+		"initial_prop" = {
+			"name" = "test"
+		}
+    }
+	"bar" = {
+		"initial_prop" = {
+			"name" = "test2"
+		}
+    }
+  }
+}
+
+variable "var2" {
+  default = {
+	"foo" = {
+		"merged_prop" = {
+			"another_name" = "test_again"
+		}
+    }
+	"bar" = {
+		"merged_prop" = {
+			"another_name" = "test_again_2"
+		}
+    }
+  }
+}
+
+locals {
+  to_merge = { for to_merge_key, to_merge_value in var.var1 : to_merge_key => {
+    merged       = module.test[local.index_prop].mod_result.out.obj
+    }
+  }
+
+  merged = { for k, v in var.var1 : k => merge(v, local.to_merge[k]) }
+}
+
+locals {
+  index_prop    =  "foo"
+}
+
+module "test" {
+  for_each = var.var2
+
+  source                      = "../."
+  name                        = each.key
+  obj                   	  = each.value.merged_prop
+}
+
+module "test2" {
+	for_each = local.merged
+  	source                      = "../."
+
+    name                        = each.key
+	obj                   	  	= each.value
+}
+`,
+		`
+variable "name" {
+	default = "?"
+}
+
+variable "obj" {
+	default = "?"
+}
+
+output "mod_result" {
+	value = {
+		"out" = {
+			"name": var.name
+			"obj": var.obj
+		}
+	}
+}
+`,
+		"",
+	)
+
+	logger := newDiscardLogger()
+	loader := modules.NewModuleLoader(filepath.Dir(path), nil, logger, &sync.KeyMutex{})
+	parsers, err := LoadParsers(path, loader, nil, logger)
+	require.NoError(t, err)
+	module, err := parsers[0].ParseDirectory()
+	require.NoError(t, err)
+
+	var mod1 *Module
+	var mod2 *Module
+	for _, m := range module.Modules {
+		if m.Name == `module.test["foo"]` {
+			mod1 = m
+		}
+
+		if m.Name == `module.test2["foo"]` {
+			mod2 = m
+		}
+	}
+
+	assert.Len(t, module.Modules, 4)
+	require.NotNil(t, mod1, "could not find module.test[foo] in root module")
+	require.NotNil(t, mod2, "could not find module.test2[foo] in root module")
+	out := mod1.Blocks.Outputs(false)
+	v := out.AsValueMap()
+	output := v["mod_result"]
+	simple := ctyJson.SimpleJSONValue{Value: output}
+	b, err := simple.MarshalJSON()
+	require.NoError(t, err)
+	assert.JSONEq(t, `{"out":{"name":"foo","obj":{"another_name":"test_again"}}}`, string(b))
+
+	out = mod2.Blocks.Outputs(false)
+	v = out.AsValueMap()
+	output = v["mod_result"]
+	simple = ctyJson.SimpleJSONValue{Value: output}
+	b, err = simple.MarshalJSON()
+	require.NoError(t, err)
+	assert.JSONEq(t, `{"out":{"name":"foo","obj":{"initial_prop":{"name":"test"},"merged":{"another_name":"test_again"}}}}`, string(b))
 }
