@@ -9,15 +9,26 @@ func getVPCEndpointRegistryItem() *schema.RegistryItem {
 	return &schema.RegistryItem{
 		Name:  "aws_vpc_endpoint",
 		RFunc: NewVPCEndpoint,
+		ReferenceAttributes: []string{
+			"subnet_ids",
+		},
 	}
 }
 
 func NewVPCEndpoint(d *schema.ResourceData, u *schema.UsageData) *schema.Resource {
-	subnetIDs := d.Get("subnet_ids").Array()
+	subnetIDs := len(d.Get("subnet_ids").Array())
 
-	interfaces := int64(1)
-	if len(subnetIDs) > 0 {
-		interfaces = int64(len(subnetIDs))
+	// if the length of the subnet_ids attribute is zero this means that the attribute
+	// has been modified with a subnet id that is yet to exist. In this instance we'll
+	// use the reference attribute instead. In most cases this should have the accurate
+	// number of subnet_ids.
+	if subnetIDs == 0 {
+		subnetIDs = len(d.References("subnet_ids"))
+	}
+
+	var interfaces int64 = 1
+	if subnetIDs > 0 {
+		interfaces = int64(subnetIDs)
 	}
 
 	r := &aws.VPCEndpoint{
