@@ -6,12 +6,11 @@ import (
 	"github.com/infracost/infracost/internal/providers/terraform/tftest"
 )
 
-func TestAzureRMVpnGateway(t *testing.T) {
+func TestVpnGatewayGoldenFile(t *testing.T) {
+	t.Parallel()
 	if testing.Short() {
 		t.Skip("skipping test in short mode")
 	}
 
-	opts := tftest.DefaultGoldenFileOptions()
-	opts.CaptureLogs = true
-	tftest.GoldenFileResourceTestsWithOpts(t, "vpn_gateway_test", opts)
+	tftest.GoldenFileResourceTests(t, "vpn_gateway_test")
 }
