@@ -7,7 +7,6 @@ import (
 	"os"
 	"path/filepath"
 	"regexp"
-	"runtime/debug"
 	"sort"
 	"strings"
 	"sync"
@@ -16,9 +15,7 @@ import (
 	"github.com/zclconf/go-cty/cty"
 	"github.com/zclconf/go-cty/cty/gocty"
 	ctyJson "github.com/zclconf/go-cty/cty/json"
-	"golang.org/x/sync/errgroup"
 
-	"github.com/infracost/infracost/internal/clierror"
 	"github.com/infracost/infracost/internal/config"
 	"github.com/infracost/infracost/internal/hcl"
 	"github.com/infracost/infracost/internal/hcl/modules"
@@ -37,7 +34,7 @@ type HCLProvider struct {
 
 	schema *PlanSchema
 	ctx    *config.ProjectContext
-	cache  []*hcl.Module
+	cache  []HCLProject
 	config HCLProviderConfig
 }
 
@@ -134,10 +131,20 @@ func NewHCLProvider(ctx *config.ProjectContext, config *HCLProviderConfig, opts
 	runCtx := ctx.RunContext
 	locatorConfig := &hcl.ProjectLocatorConfig{ExcludedSubDirs: ctx.ProjectConfig.ExcludePaths, ChangedObjects: runCtx.VCSMetadata.Commit.ChangedObjects, UseAllPaths: ctx.ProjectConfig.IncludeAllPaths}
 
-	path := ctx.RunContext.Config.RepoPath()
-	loader := modules.NewModuleLoader(path, credsSource, logger, ctx.RunContext.ModuleMutex)
+	wd := ctx.RunContext.Config.RepoPath()
+	initialPath := ctx.ProjectConfig.Path
+	if filepath.IsAbs(wd) {
+		abs, err := filepath.Abs(initialPath)
+		if err != nil {
+			logger.WithError(err).Warnf("could not make project path absolute to match provided --config-file/--path path absolute, this will result in module loading failures")
+		} else {
+			initialPath = abs
+		}
+	}
+
+	loader := modules.NewModuleLoader(wd, credsSource, logger, ctx.RunContext.ModuleMutex)
 	parsers, err := hcl.LoadParsers(
-		ctx.ProjectConfig.Path,
+		initialPath,
 		loader,
 		locatorConfig,
 		logger,
@@ -154,7 +161,7 @@ func NewHCLProvider(ctx *config.ProjectContext, config *HCLProviderConfig, opts
 	return &HCLProvider{
 		scanner:        scanner,
 		parsers:        parsers,
-		planJSONParser: NewParser(ctx, false),
+		planJSONParser: NewParser(ctx, true),
 		ctx:            ctx,
 		config:         *config,
 		logger:         logger,
@@ -164,6 +171,8 @@ func NewHCLProvider(ctx *config.ProjectContext, config *HCLProviderConfig, opts
 func (p *HCLProvider) Type() string        { return "terraform_dir" }
 func (p *HCLProvider) DisplayType() string { return "Terraform directory" }
 func (p *HCLProvider) AddMetadata(metadata *schema.ProjectMetadata) {
+	metadata.ConfigSha = p.ctx.ProjectConfig.ConfigSha
+
 	basePath := p.ctx.ProjectConfig.Path
 	if p.ctx.RunContext.Config.ConfigFilePath != "" {
 		basePath = filepath.Dir(p.ctx.RunContext.Config.ConfigFilePath)
@@ -182,18 +191,16 @@ func (p *HCLProvider) AddMetadata(metadata *schema.ProjectMetadata) {
 // representation of the terraform plan JSON files from these Blocks, this is passed to the PlanJSONProvider.
 // The PlanJSONProvider uses this shallow representation to actually load Infracost resources.
 func (p *HCLProvider) LoadResources(usage map[string]*schema.UsageData) ([]*schema.Project, error) {
-	jsons, err := p.LoadPlanJSONs()
-	if err != nil {
-		return nil, err
-	}
+	jsons := p.LoadPlanJSONs()
 
 	var projects = make([]*schema.Project, len(jsons))
 	for i, j := range jsons {
-		project, err := p.parseResources(j, usage)
-		if err != nil {
-			return nil, err
+		if j.Error != nil {
+			projects[i] = p.newProject(j)
+			continue
 		}
 
+		project := p.parseResources(j, usage)
 		if p.ctx.RunContext.VCSMetadata.HasChanges() {
 			project.Metadata.VCSCodeChanged = &j.Module.HasChanges
 		}
@@ -210,18 +217,25 @@ func (p *HCLProvider) LoadResources(usage map[string]*schema.UsageData) ([]*sche
 	return projects, nil
 }
 
-func (p *HCLProvider) parseResources(parsed HCLProject, usage map[string]*schema.UsageData) (*schema.Project, error) {
+func (p *HCLProvider) parseResources(parsed HCLProject, usage map[string]*schema.UsageData) *schema.Project {
 	project := p.newProject(parsed)
 
 	partialPastResources, partialResources, err := p.planJSONParser.parseJSON(parsed.JSON, usage)
 	if err != nil {
-		return project, fmt.Errorf("Error parsing Terraform plan JSON file %w", err)
+		project.Metadata.Errors = []schema.ProjectDiag{
+			{
+				Code:    schema.DiagJSONParsingFailure,
+				Message: err.Error(),
+			},
+		}
+
+		return project
 	}
 
 	project.PartialPastResources = partialPastResources
 	project.PartialResources = partialResources
 
-	return project, nil
+	return project
 }
 
 func (p *HCLProvider) newProject(parsed HCLProject) *schema.Project {
@@ -229,11 +243,20 @@ func (p *HCLProvider) newProject(parsed HCLProject) *schema.Project {
 	metadata.Type = p.Type()
 	p.AddMetadata(metadata)
 
+	if parsed.Error != nil {
+		metadata.Errors = []schema.ProjectDiag{
+			{
+				Code:    schema.DiagModuleEvaluationFailure,
+				Message: parsed.Error.Error(),
+			},
+		}
+	}
+
 	if len(parsed.Module.Warnings) > 0 {
-		warnings := make([]schema.Warning, len(parsed.Module.Warnings))
+		warnings := make([]schema.ProjectDiag, len(parsed.Module.Warnings))
 
 		for i, warning := range parsed.Module.Warnings {
-			warnings[i] = schema.Warning{
+			warnings[i] = schema.ProjectDiag{
 				Code:    int(warning.Code),
 				Message: warning.Title,
 				Data:    warning.Data,
@@ -256,47 +279,43 @@ func (p *HCLProvider) newProject(parsed HCLProject) *schema.Project {
 type HCLProject struct {
 	JSON   []byte
 	Module *hcl.Module
+	Error  error
 }
 
 // LoadPlanJSONs parses the found directories and return the blocks in Terraform plan JSON format.
-func (p *HCLProvider) LoadPlanJSONs() ([]HCLProject, error) {
+func (p *HCLProvider) LoadPlanJSONs() []HCLProject {
 	var jsons = make([]HCLProject, len(p.parsers))
-	modules, err := p.Modules()
-	if err != nil {
-		return nil, err
-	}
+	mods := p.Modules()
+
+	for i, module := range mods {
+		if module.Error == nil {
+			b, err := p.modulesToPlanJSON(module.Module)
+			if err != nil {
+				module.Error = err
+			} else {
+				module.JSON = b
+			}
 
-	for i, module := range modules {
-		b, err := p.modulesToPlanJSON(module)
-		if err != nil {
-			return nil, err
 		}
 
-		jsons[i] = HCLProject{
-			JSON:   b,
-			Module: module,
-		}
+		jsons[i] = module
 	}
 
-	return jsons, nil
+	return jsons
 }
 
 // Modules parses the found directories into hcl modules representing a config tree of Terraform information.
 // Modules returns the raw hcl blocks associated with each found Terraform project. This can be used
 // to fetch raw information like outputs, vars, resources, e.t.c.
-func (p *HCLProvider) Modules() ([]*hcl.Module, error) {
+func (p *HCLProvider) Modules() []HCLProject {
 	if p.cache != nil {
-		return p.cache, nil
+		return p.cache
 	}
 
 	runCtx := p.ctx.RunContext
-	parallelism, err := runCtx.GetParallelism()
-	if err != nil {
-		return nil, err
-	}
+	parallelism, _ := runCtx.GetParallelism()
 
 	numJobs := len(p.parsers)
-
 	runInParallel := parallelism > 1 && numJobs > 1
 	if runInParallel && !runCtx.Config.IsLogging() {
 		if runInParallel && !p.config.SuppressLogging {
@@ -309,61 +328,56 @@ func (p *HCLProvider) Modules() ([]*hcl.Module, error) {
 		p.ctx.RunContext.Config.LogLevel = "info"
 	}
 
-	var mods = make([]*hcl.Module, 0, len(p.parsers))
-	errGroup := &errgroup.Group{}
-	ch := make(chan *hcl.Parser, len(p.parsers))
+	if numJobs < parallelism {
+		parallelism = numJobs
+	}
+
+	ch := make(chan *hcl.Parser, numJobs)
+	mods := make([]HCLProject, 0, numJobs)
+	mu := &sync.Mutex{}
+	wg := &sync.WaitGroup{}
+
 	for _, parser := range p.parsers {
 		ch <- parser
 	}
 	close(ch)
-	lock := &sync.Mutex{}
+	wg.Add(parallelism)
 
 	for i := 0; i < parallelism; i++ {
-		errGroup.Go(func() (err error) {
+		go func() {
 			defer func() {
-				e := recover()
-				if e != nil {
-					err = clierror.NewPanicError(fmt.Errorf("%s", e), debug.Stack())
-				}
+				wg.Done()
 			}()
 
 			for parser := range ch {
-				if len(p.parsers) > 1 && !p.config.SuppressLogging {
+				if numJobs > 1 && !p.config.SuppressLogging {
 					fmt.Fprintf(os.Stderr, "Detected Terraform project at %s\n", ui.DisplayPath(parser.Path()))
 				}
 
 				module, err := parser.ParseDirectory()
-				if err != nil {
-					return err
-				}
 
-				lock.Lock()
-				mods = append(mods, module)
-				lock.Unlock()
+				mu.Lock()
+				mods = append(mods, HCLProject{Module: module, Error: err})
+				mu.Unlock()
 			}
-
-			return nil
-		})
+		}()
 	}
 
-	err = errGroup.Wait()
-	if err != nil {
-		return nil, err
-	}
+	wg.Wait()
 
 	if p.config.CacheParsingModules {
 		p.cache = mods
 	}
 
 	sort.Slice(mods, func(i, j int) bool {
-		if mods[i].Name != "" && mods[j].Name != "" {
-			return mods[i].Name < mods[j].Name
+		if mods[i].Module.Name != "" && mods[j].Module.Name != "" {
+			return mods[i].Module.Name < mods[j].Module.Name
 		}
 
-		return mods[i].ModulePath < mods[j].ModulePath
+		return mods[i].Module.ModulePath < mods[j].Module.ModulePath
 	})
 
-	return mods, nil
+	return mods
 }
 
 // InvalidateCache removes the module cache from the prior hcl parse.
@@ -378,15 +392,23 @@ func (p *HCLProvider) newPlanSchema() {
 		FormatVersion:    "1.0",
 		TerraformVersion: "1.1.0",
 		Variables:        nil,
-		PlannedValues: struct {
-			RootModule PlanModule `json:"root_module"`
+		PriorState: struct {
+			Values PlanValues `json:"values"`
 		}{
+			Values: PlanValues{
+				RootModule: PlanModule{
+					Resources:    []ResourceJSON{},
+					ChildModules: []PlanModule{},
+				},
+			},
+		},
+		InfracostResourceChanges: []ResourceChangesJSON{},
+		PlannedValues: PlanValues{
 			RootModule: PlanModule{
 				Resources:    []ResourceJSON{},
 				ChildModules: []PlanModule{},
 			},
 		},
-		ResourceChanges: []ResourceChangesJSON{},
 		Configuration: Configuration{
 			ProviderConfig: make(map[string]ProviderConfig),
 			RootModule: ModuleConfig{
@@ -402,12 +424,14 @@ func (p *HCLProvider) modulesToPlanJSON(rootModule *hcl.Module) ([]byte, error)
 
 	mo := p.marshalModule(rootModule)
 	p.schema.Configuration.RootModule = mo.ModuleConfig
+	p.schema.PriorState.Values.RootModule = mo.PlanModule
 	p.schema.PlannedValues.RootModule = mo.PlanModule
 
 	b, err := json.MarshalIndent(p.schema, "", "  ")
 	if err != nil {
 		return nil, fmt.Errorf("error handling built plan json from hcl %w", err)
 	}
+
 	return b, nil
 }
 
@@ -438,8 +462,7 @@ func (p *HCLProvider) marshalModule(module *hcl.Module) ModuleOut {
 			}
 
 			planModule.Resources = append(planModule.Resources, out.Planned)
-
-			p.schema.ResourceChanges = append(p.schema.ResourceChanges, out.Changes)
+			p.schema.InfracostResourceChanges = append(p.schema.InfracostResourceChanges, out.Changes)
 		}
 	}
 
@@ -492,8 +515,8 @@ func (p *HCLProvider) getResourceOutput(block *hcl.Block) ResourceOutput {
 	jsonValues := marshalAttributeValues(block.Type(), block.Values())
 	p.marshalBlock(block, jsonValues)
 
-	changes.Change.After = jsonValues
 	planned.Values = jsonValues
+	changes.Change.After = jsonValues
 
 	providerConfigKey := strings.Split(block.TypeLabel(), "_")[0]
 
@@ -535,6 +558,7 @@ func (p *HCLProvider) getResourceOutput(block *hcl.Block) ResourceOutput {
 
 	return ResourceOutput{
 		Planned:       planned,
+		PriorState:    planned,
 		Changes:       changes,
 		Configuration: configuration,
 	}
@@ -689,6 +713,7 @@ func marshalAttributeValues(blockType string, value cty.Value) map[string]interf
 type ResourceOutput struct {
 	Planned       ResourceJSON
 	Changes       ResourceChangesJSON
+	PriorState    ResourceJSON
 	Configuration ResourceData
 }
 
@@ -719,15 +744,25 @@ type ResourceChange struct {
 	After   map[string]interface{} `json:"after"`
 }
 
+type PlanValues struct {
+	RootModule PlanModule `json:"root_module"`
+}
+
 type PlanSchema struct {
 	FormatVersion    string      `json:"format_version"`
 	TerraformVersion string      `json:"terraform_version"`
 	Variables        interface{} `json:"variables,omitempty"`
-	PlannedValues    struct {
-		RootModule PlanModule `json:"root_module"`
-	} `json:"planned_values"`
-	ResourceChanges []ResourceChangesJSON `json:"resource_changes"`
-	Configuration   Configuration         `json:"configuration"`
+	PriorState       struct {
+		Values PlanValues `json:"values"`
+	} `json:"prior_state"`
+	PlannedValues PlanValues    `json:"planned_values"`
+	Configuration Configuration `json:"configuration"`
+
+	// InfracostResourceChanges is a flattened list of resource changes for the plan, this is in the format of the Terraform
+	// plan JSON output, but we omit adding it as the supported `resource_changes` key as this will cause plan inconsistencies.
+	// We copy this `infracost_resource_changes` key at a later date to `resource_changes` before sending to the Policy API.
+	// This means that we can evaluate the Rego ruleset on the known Terraform plan JSON structure.
+	InfracostResourceChanges []ResourceChangesJSON `json:"infracost_resource_changes"`
 }
 
 type PlanModule struct {
