@@ -310,13 +310,7 @@ func loadResources(t *testing.T, pName string, tfProject TerraformProject, runCt
 	require.NoError(t, err)
 
 	for _, project := range projects {
-		for _, partial := range project.PartialResources {
-			project.Resources = append(project.Resources, schema.BuildResource(partial, nil))
-		}
-
-		for _, partial := range project.PartialPastResources {
-			project.PastResources = append(project.PastResources, schema.BuildResource(partial, nil))
-		}
+		project.BuildResources(map[string]*schema.UsageData{})
 	}
 
 	return projects
