@@ -13,22 +13,39 @@ import (
 	"github.com/infracost/infracost/internal/vcs"
 )
 
-// Warning holds information about non-critical errors that occurred when evaluating a project.
-type Warning struct {
+const (
+	DiagJSONParsingFailure = iota + 1
+	DiagModuleEvaluationFailure
+	DiagTerragruntEvaluationFailure
+)
+
+// ProjectDiag holds information about all diagnostics associated with a project.
+// This can be both critical or warnings.
+type ProjectDiag struct {
 	Code    int         `json:"code"`
 	Message string      `json:"message"`
 	Data    interface{} `json:"data"`
 }
 
 type ProjectMetadata struct {
-	Path                string    `json:"path"`
-	Type                string    `json:"type"`
-	TerraformModulePath string    `json:"terraformModulePath,omitempty"`
-	TerraformWorkspace  string    `json:"terraformWorkspace,omitempty"`
-	VCSSubPath          string    `json:"vcsSubPath,omitempty"`
-	VCSCodeChanged      *bool     `json:"vcsCodeChanged,omitempty"`
-	Warnings            []Warning `json:"warnings,omitempty"`
-	Policies            Policies  `json:"policies,omitempty"`
+	Path                string        `json:"path"`
+	Type                string        `json:"type"`
+	ConfigSha           string        `json:"configSha,omitempty"`
+	TerraformModulePath string        `json:"terraformModulePath,omitempty"`
+	TerraformWorkspace  string        `json:"terraformWorkspace,omitempty"`
+	VCSSubPath          string        `json:"vcsSubPath,omitempty"`
+	VCSCodeChanged      *bool         `json:"vcsCodeChanged,omitempty"`
+	Errors              []ProjectDiag `json:"errors,omitempty"`
+	Warnings            []ProjectDiag `json:"warnings,omitempty"`
+	Policies            Policies      `json:"policies,omitempty"`
+}
+
+func (m *ProjectMetadata) AddError(err error) {
+	m.Errors = append(m.Errors, ProjectDiag{Message: err.Error()})
+}
+
+func (m *ProjectMetadata) HasErrors() bool {
+	return len(m.Errors) > 0
 }
 
 func (m *ProjectMetadata) WorkspaceLabel() string {
@@ -104,20 +121,75 @@ func (p *Project) NameWithWorkspace() string {
 
 // AllResources returns a pointer list of all resources of the state.
 func (p *Project) AllResources() []*Resource {
+	m := make(map[*Resource]bool)
+	for _, r := range p.PastResources {
+		m[r] = true
+	}
+
+	for _, r := range p.Resources {
+		if _, ok := m[r]; !ok {
+			m[r] = true
+		}
+	}
+
 	var resources []*Resource
-	resources = append(resources, p.PastResources...)
-	resources = append(resources, p.Resources...)
+	for r := range m {
+		resources = append(resources, r)
+	}
+
 	return resources
 }
 
 // AllPartialResources returns a pointer list of the current and past partial resources
 func (p *Project) AllPartialResources() []*PartialResource {
+	m := make(map[*PartialResource]bool)
+	for _, r := range p.PartialPastResources {
+		m[r] = true
+	}
+
+	for _, r := range p.PartialResources {
+		if _, ok := m[r]; !ok {
+			m[r] = true
+		}
+	}
+
 	var resources []*PartialResource
-	resources = append(resources, p.PartialPastResources...)
-	resources = append(resources, p.PartialResources...)
+	for r := range m {
+		resources = append(resources, r)
+	}
+
 	return resources
 }
 
+// BuildResources builds the resources from the partial resources
+// and sets the PastResources and Resources fields.
+func (p *Project) BuildResources(usageMap map[string]*UsageData) {
+	pastResources := make([]*Resource, 0, len(p.PartialPastResources))
+	resources := make([]*Resource, 0, len(p.PartialResources))
+
+	seen := make(map[*PartialResource]*Resource)
+
+	for _, p := range p.PartialPastResources {
+		u := usageMap[p.ResourceData.Address]
+		r := BuildResource(p, u)
+		seen[p] = r
+		pastResources = append(pastResources, r)
+	}
+
+	for _, p := range p.PartialResources {
+		r, ok := seen[p]
+		if !ok {
+			u := usageMap[p.ResourceData.Address]
+			r = BuildResource(p, u)
+			seen[p] = r
+		}
+		resources = append(resources, r)
+	}
+
+	p.PastResources = pastResources
+	p.Resources = resources
+}
+
 // CalculateDiff calculates the diff of past and current resources
 func (p *Project) CalculateDiff() {
 	if p.HasDiff {
