@@ -1,10 +1,14 @@
 package schema
 
 import (
+	"encoding/json"
 	"fmt"
+	"math"
 	"strings"
 
+	"github.com/imdario/mergo"
 	jsoniter "github.com/json-iterator/go"
+	log "github.com/sirupsen/logrus"
 	"github.com/tidwall/gjson"
 )
 
@@ -20,6 +24,35 @@ func NewUsageData(address string, attributes map[string]gjson.Result) *UsageData
 	}
 }
 
+// Merge returns a new UsageData which is the result of adding all keys from other that do not already exists in the usage data
+func (u *UsageData) Merge(other *UsageData) *UsageData {
+	if u == nil {
+		if other != nil {
+			return other.Merge(u) // this will return a new copy of other
+		}
+		return nil // both are nil
+	}
+
+	newU := &UsageData{
+		Address:    u.Address,
+		Attributes: make(map[string]gjson.Result, len(u.Attributes)),
+	}
+
+	for k, v := range u.Attributes {
+		newU.Attributes[k] = v
+	}
+
+	if other != nil {
+		for k, v := range other.Attributes {
+			if _, ok := newU.Attributes[k]; !ok {
+				newU.Attributes[k] = v
+			}
+		}
+	}
+
+	return newU
+}
+
 func (u *UsageData) Get(key string) gjson.Result {
 	if u.Attributes[key].Type != gjson.Null {
 		return u.Attributes[key]
@@ -42,6 +75,10 @@ func (u *UsageData) GetFloat(key string) *float64 {
 func (u *UsageData) GetInt(key string) *int64 {
 	if u.Get(key).Type != gjson.Null {
 		val := u.Get(key).Int()
+		if val == 0 {
+			fVal := u.Get(key).Float()
+			val = int64(math.Floor(fVal))
+		}
 		return &val
 	}
 
@@ -149,3 +186,50 @@ func ParseAttributes(i interface{}) map[string]gjson.Result {
 
 	return a
 }
+
+func MergeAttributes(dst *UsageData, src *UsageData) {
+	for key, srcAttr := range src.Attributes {
+		if _, has := dst.Attributes[key]; has {
+			switch srcAttr.Type {
+			case gjson.Null:
+				fallthrough
+			case gjson.True:
+				fallthrough
+			case gjson.False:
+				fallthrough
+			case gjson.Number:
+				fallthrough
+			case gjson.String:
+				// Should be safe to override
+				dst.Attributes[key] = srcAttr
+			case gjson.JSON:
+				var err error
+				var destJson map[string]interface{}
+				var srcJson map[string]interface{}
+				err = json.Unmarshal([]byte(dst.Attributes[key].Raw), &destJson)
+				if err != nil {
+					log.Errorf("Error merging attribute '%s': %v", key, err)
+					break
+				}
+				err = json.Unmarshal([]byte(srcAttr.Raw), &srcJson)
+				if err != nil {
+					log.Errorf("Error merging attribute '%s': %v", key, err)
+					break
+				}
+				err = mergo.Map(&destJson, srcJson)
+				if err != nil {
+					log.Errorf("Error merging attribute '%s': %v", key, err)
+					break
+				}
+				src, err := json.Marshal(destJson)
+				if err != nil {
+					log.Errorf("Error merging attribute '%s': %v", key, err)
+					break
+				}
+				dst.Attributes[key] = gjson.Parse(string(src))
+			}
+		} else {
+			dst.Attributes[key] = srcAttr
+		}
+	}
+}
