@@ -13,7 +13,6 @@ import (
 	"sort"
 	"strconv"
 	"strings"
-	"sync"
 	"time"
 
 	"github.com/go-git/go-git/v5"
@@ -24,6 +23,7 @@ import (
 	"github.com/tidwall/gjson"
 
 	"github.com/infracost/infracost/internal/logging"
+	"github.com/infracost/infracost/internal/sync"
 )
 
 var (
@@ -249,30 +249,18 @@ func envToTime(key string) time.Time {
 	return t.UTC()
 }
 
-type keyMutex struct {
-	mutexes sync.Map // Zero value is empty and ready for use
-}
-
-func (m *keyMutex) Lock(key string) func() {
-	value, _ := m.mutexes.LoadOrStore(key, &sync.Mutex{})
-	mtx := value.(*sync.Mutex)
-	mtx.Lock()
-
-	return func() { mtx.Unlock() }
-}
-
 // metadataFetcher is an object designed to find metadata for different systems.
 // It is designed to be safe for parallelism. So interactions across branches and for different commits
 // will not affect other goroutines.
 type metadataFetcher struct {
-	mu     *keyMutex
+	mu     *sync.KeyMutex
 	client *http.Client
 	test   *bool
 }
 
 func newMetadataFetcher() *metadataFetcher {
 	return &metadataFetcher{
-		mu:     &keyMutex{},
+		mu:     &sync.KeyMutex{},
 		client: &http.Client{Timeout: time.Second * 5},
 	}
 }
@@ -644,8 +632,8 @@ func (f *metadataFetcher) getAzureReposMetadata(path string, gitDiffTarget *stri
 		VCSProvider:  "azure_devops_tfsgit",
 		Title:        res.Title,
 		Author:       res.CreatedBy.UniqueName,
-		SourceBranch: strings.TrimLeft(getEnv("SYSTEM_PULLREQUEST_SOURCEBRANCH"), "refs/heads/"),
-		BaseBranch:   strings.TrimLeft(getEnv("SYSTEM_PULLREQUEST_TARGETBRANCH"), "refs/heads/"),
+		SourceBranch: strings.TrimPrefix(getEnv("SYSTEM_PULLREQUEST_SOURCEBRANCH"), "refs/heads/"),
+		BaseBranch:   strings.TrimPrefix(getEnv("SYSTEM_PULLREQUEST_TARGETBRANCH"), "refs/heads/"),
 		URL:          fmt.Sprintf("%s/pullrequest/%s", getEnv("SYSTEM_PULLREQUEST_SOURCEREPOSITORYURI"), pullID),
 	}
 	return m, nil
@@ -705,6 +693,7 @@ func (f *metadataFetcher) getAzureReposPRInfo() azurePullRequestResponse {
 		logging.Logger.WithError(err).Debugf("could not fetch Azure DevOps pull request information using URL '%s'", apiURL)
 		return out
 	}
+	defer res.Body.Close()
 
 	if res.StatusCode != http.StatusOK {
 		b, _ := io.ReadAll(res.Body)
@@ -742,8 +731,8 @@ func (f *metadataFetcher) getAzureReposGithubMetadata(path string, gitDiffTarget
 	m.PullRequest = &PullRequest{
 		ID:           pullNumber,
 		VCSProvider:  "azure_devops_github",
-		SourceBranch: strings.TrimLeft(getEnv("SYSTEM_PULLREQUEST_SOURCEBRANCH"), "refs/heads/"),
-		BaseBranch:   strings.TrimLeft(getEnv("SYSTEM_PULLREQUEST_TARGETBRANCH"), "refs/heads/"),
+		SourceBranch: strings.TrimPrefix(getEnv("SYSTEM_PULLREQUEST_SOURCEBRANCH"), "refs/heads/"),
+		BaseBranch:   strings.TrimPrefix(getEnv("SYSTEM_PULLREQUEST_TARGETBRANCH"), "refs/heads/"),
 		URL:          fmt.Sprintf("%s/pulls/%s", getEnv("SYSTEM_PULLREQUEST_SOURCEREPOSITORYURI"), pullNumber),
 
 		// We are unable to fetch pull request title and author for repositories using the GitHub <> Azure DevOps
@@ -761,7 +750,7 @@ func (f *metadataFetcher) getAzureReposGithubMetadata(path string, gitDiffTarget
 // in the original Commit of Metadata m. Otherwise, transformAzureDevOpsMergeCommit returns the first commit
 // on a git log call that doesn't appear to be a Merge commit.
 func (f *metadataFetcher) transformAzureDevOpsMergeCommit(path string, m *Metadata) error {
-	m.Branch.Name = strings.TrimLeft(getEnv("SYSTEM_PULLREQUEST_SOURCEBRANCH"), "refs/heads/")
+	m.Branch.Name = strings.TrimPrefix(getEnv("SYSTEM_PULLREQUEST_SOURCEBRANCH"), "refs/heads/")
 
 	matches := mergeCommitRegxp.FindStringSubmatch(m.Commit.Message)
 	if len(matches) <= 1 {
@@ -1055,7 +1044,7 @@ func urlStringToRemote(remote string) Remote {
 	}
 
 	if strings.Contains(host, "azure") {
-		host = strings.TrimLeft(m[2], "ssh.")
+		host = strings.TrimPrefix(m[2], "ssh.")
 		path = versionRegxp.ReplaceAllString(path, "")
 		pieces := strings.Split(path, "/")
 		if len(pieces) == 3 {
